unit main;

interface

uses   
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, ShellAPI,
  Dialogs, DSUtil, StdCtrls, DSPack, DirectShow9, Menus, ExtCtrls, INIfiles, Jpeg;

type
  TFileInfo = record
    Icon : hIcon;
    Image : Integer;
    DisplayName : String;
    TypeName : String;
    Size : Integer;
    SizeDescription : String;
    DateTime : TDateTime;
    AttrArchive : Boolean;
    AttrReadOnly : Boolean;
    AttrSystem : Boolean;
    AttrHidden : Boolean;
    AttrVolume : Boolean;
    AttrDirectory : Boolean;
  end;

type
  TVideoForm = class(TForm)
    FilterGraph: TFilterGraph;
    VideoWindow: TVideoWindow;
    Filter: TFilter;
    Image: TImage;
    SampleGrabber: TSampleGrabber;
    SnapShot: TButton;
    CallBack: TCheckBox;
    lInfo: TLabel;
    btnOff: TButton;
    btnOn: TButton;
    Timer1: TTimer;
    lblb: TLabel;
    Label1: TLabel;
    lInfoTimer: TLabel;
    lPath: TLabel;
    Timer2: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SnapShotClick(Sender: TObject);
    procedure SampleGrabberBuffer(sender: TObject; SampleTime: Double;
      pBuffer: Pointer; BufferLen: Integer);
    procedure OffCamera();
    procedure Nacti();
    procedure btnOnClick(Sender: TObject);
    procedure btnOffClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ZapisInfo(path: string; pict: string);
    procedure BmpToJpeg(Bmp: TBitmap; Q: TJpegQualityRange; var Jpg: TJpegImage);
    procedure Timer2Timer(Sender: TObject);
    procedure scGetFileInfo(StrPath : String; var Info : TFileInfo);

    function ReadMWord(f: TFileStream): Word;
    procedure GetJPGSize(const sFile: string; var wWidth, wHeight: Word);
  private
    { Dclarations prives }
  public
    { Dclarations publiques }
    procedure OnSelectDevice(Sender: TObject);
    procedure WMSysCommand(var Message: TMessage); message wm_SysCommand;
  end;



const
  idSysInfo = 100;
  UnixStartDate : tdatetime = 719163.0;

var
  VideoForm: TVideoForm;
  SysDev: TSysDevEnum;
  hSysMenu: hMenu;

  index: integer;
  img_path: string;
  jmeno: string;
  snimkovani: boolean;
  onepict: boolean;
  dec : integer;


implementation

{$R *.dfm}

uses setting;

procedure TVideoForm.WMSysCommand(var Message: TMessage);
var
  frmSet : TfrmSet;
begin

  if Message.WParam = idSysInfo then
  begin

        frmSet := TfrmSet.Create(self);
        try
           frmSet.ShowModal;
        finally
           FrmSet.Free;
           Nacti();
        end;

  end;
  inherited;
end;

procedure TVideoForm.Nacti();
var
  ini : TINIFile;
  ontop : Boolean;
  interval : integer;
begin

  ini := TIniFile.Create(ExtractFilePath(application.ExeName) + 'setting.ini');

      img_path := ini.ReadString('SNIMKY','Cesta','');
      jmeno := ini.ReadString('SNIMKY','Jmeno','obrazek');
      lPath.Caption := img_path;


      interval := StrToInt(ini.ReadString('SNIMKY','Interval','15'));
      Timer1.Interval := interval * 1000;
      dec := interval;


      onepict := ini.ReadBool('SNIMKY','JedenSnimek',true);
      snimkovani := ini.ReadBool('SNIMKY','SerieSnimku',false);

      ontop := ini.ReadBool('APP','VzdyNaVrchu',false);
      if ontop = true then
        VideoForm.FormStyle := fsStayOnTop;

      VideoForm.Top := ini.ReadInteger('APP','windows_top', 50 );
      VideoForm.Left := ini.ReadInteger('APP','windows_left', 50 );



      index := ini.ReadInteger('KAMERA','Index',0);

  ini.Free;

end;

procedure TVideoForm.FormCreate(Sender: TObject);

begin


  Nacti();

  hSysMenu := GetSystemMenu(VideoForm.Handle, False);
  AppendMenu(hSysMenu, MF_SEPARATOR, 0, '');
  AppendMenu(hSysMenu, MF_STRING, idSysInfo, 'Nastaveni');


  SysDev:= TSysDevEnum.Create(CLSID_VideoInputDeviceCategory);
  if SysDev.CountFilters > 0 then

    OnSelectDevice(sender);

  lInfoTimer.Caption := '?';
  lPath.Caption := img_path;

end;

procedure TVideoForm.OnSelectDevice(Sender: TObject);
begin

     FilterGraph.ClearGraph;
     FilterGraph.Active := false;
     Filter.BaseFilter.Moniker := SysDev.GetMoniker(index);
     FilterGraph.Active := true;
     with FilterGraph as ICaptureGraphBuilder2 do
       RenderStream(@PIN_CATEGORY_PREVIEW, nil, Filter as IBaseFilter, SampleGrabber as IBaseFilter, VideoWindow as IbaseFilter);
     FilterGraph.Play;

     btnOn.Enabled := False;
     btnOff.Enabled := True;
     SnapShot.Enabled := True;


     Sleep(1000);
     SnapShotClick(Sender);


end;

procedure TVideoForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
 ini : TINIFile;
begin
  CallBack.Checked := False;
  SysDev.Free;
  FilterGraph.ClearGraph;
  FilterGraph.Active := false;

  ini := TIniFile.Create(ExtractFilePath(application.ExeName) + 'Setting.ini');
    ini.WriteInteger('APP','windows_top', VideoForm.Top  );
    ini.WriteInteger('APP','windows_left', VideoForm.Left );
    ini.UpdateFile;
  ini.Free;

end;

procedure TVideoForm.OffCamera();
begin
  CallBack.Checked := False;
  FilterGraph.ClearGraph;
  FilterGraph.Active := false;


  lInfo.Caption := 'Kamera je vypnuta';
  btnOff.Enabled := False;
  btnOn.Enabled := True;
  SnapShot.Enabled := False;

end;


procedure TVideoForm.SnapShotClick(Sender: TObject);
var
  cesta: string;
  cas: string;
  interval : integer;
  unix : longint;
  aaaa : TDateTime;
  JPG : TJpegImage;
begin

  JPG := TJpegImage.Create;

  lInfo.Caption := FormatDateTime('d.m.YYYY h:nn:ss', Now);
  SampleGrabber.GetBitmap(Image.Picture.Bitmap);

  Image.Picture.Bitmap.Canvas.Font.Color := clLime;
  Image.Picture.Bitmap.Canvas.Brush.Color := clBlack;
  Image.Picture.Bitmap.Canvas.Font.Size  := 8;
  Image.Picture.Bitmap.Canvas.Font.Style := [fsBold];
  Image.Picture.Bitmap.Canvas.TextOut(0,0,lInfo.Caption);

  BmpToJpeg(Image.Picture.Bitmap, 80, JPG);


  dec := Timer1.Interval div 1000;

  if onepict = true then
     begin
        cesta := jmeno + '.jpg';
        if img_path <> '' then cesta := img_path + '\' + cesta;
        JPG.SaveToFile(cesta);
     end;

  if snimkovani = true then
     begin
        cas := FormatDateTime('YYYYmmddhhnnss', Now);
        cesta := jmeno + '-' + cas + '.jpg';
        if img_path <> '' then cesta := img_path + '\' + cesta;
        JPG.SaveToFile(cesta);
     end;

  JPG.Free;

//  Image.Picture.SaveToFile(cesta);

  lPath.Caption := img_path;

  if img_path <> '' then ZapisInfo(img_path+'/',cesta)
  else ZapisInfo('',cesta);

end;

procedure TVideoForm.ZapisInfo(path: string; pict: string);
var
  myFile : TextFile;
  finfo : TFileInfo;
  imgw, imgh : word;
begin

  scGetFileInfo(pict,finfo);
  GetJPGSize(pict,imgw,imgh);


  AssignFile(myFile, path +'webcam.xml');
  ReWrite(myFile);

  WriteLn(myFile, SysDev.Filters[index].FriendlyName );
  WriteLn(myFile, FormatDateTime('d.m.YYYY h:nn:ss', Now) );
  WriteLn(myFile, pict );
  WriteLn(myFile, jmeno );
  WriteLn(myFile, IntToStr(finfo.Size));
  WriteLn(myFile, IntToStr(imgw));
  WriteLn(myFile, IntToStr(imgh));
  WriteLn(myFile, 'JPEG' );

  if snimkovani = true then
     WriteLn(myFile, 'archivovano')
  else
     WriteLn(myFile, 'bez archivace');

  WriteLn(myFile, IntToStr( Timer1.Interval div 1000 ) );

  CloseFile(myFile);

end;


procedure TVideoForm.SampleGrabberBuffer(sender: TObject;
  SampleTime: Double; pBuffer: Pointer; BufferLen: Integer);
begin
  if CallBack.Checked then
  begin
    Image.Canvas.Lock;
    try
      SampleGrabber.GetBitmap(Image.Picture.Bitmap, pBuffer, BufferLen);
    finally
      Image.Canvas.Unlock;
    end;
  end;
end;

procedure TVideoForm.btnOnClick(Sender: TObject);
begin

   OnSelectDevice(Sender);
   Timer1.Enabled := True;

end;

procedure TVideoForm.btnOffClick(Sender: TObject);
begin

  OffCamera;
  Timer1.Enabled := False;

end;

procedure TVideoForm.Timer1Timer(Sender: TObject);
begin

  SnapShotClick(Sender);


end;

procedure TVideoForm.BmpToJpeg(Bmp: TBitmap; Q: TJpegQualityRange; var Jpg: TJpegImage);
begin
Jpg.CompressionQuality := Q;
Jpg.Assign(Bmp);
end;


procedure TVideoForm.Timer2Timer(Sender: TObject);
begin
    dec := dec - 1;
    lInfoTimer.Caption := IntToStr(dec) + ' sekund';

end;

// ----------------------------------------------------------------
// Return string with formatted file size (bytes, Kb, Mb or Gb)
// ----------------------------------------------------------------
function scGetSizeDescription(const IntSize : Int64) : String;
begin
  if IntSize < 1024 then
    Result := IntToStr(IntSize)+' bytes'
  else
  begin
    if IntSize < (1024 * 1024) then
      Result := FormatFloat('####0.##',IntSize / 1024)+' Kb'
    else
      if IntSize < (1024 * 1024 * 1024) then
        Result := FormatFloat('####0.##',IntSize / 1024 / 1024)+' Mb'
      else
        Result := FormatFloat('####0.##',IntSize / 1024 / 1024 / 1024)+' Gb';
  end;
end;

// ----------------------------------------------------------------
// Return record with all information about given file
// How to use icon : ImageFile.Picture.Icon.Handle:=Info.Icon;
// ----------------------------------------------------------------
procedure TVideoForm.scGetFileInfo(StrPath : String; var Info : TFileInfo);
var
  SHFileInfo : TSHFileInfo;
  SearchRec : TSearchRec;
begin
  if Trim(StrPath) = '' then
    Exit;

  ShGetFileInfo(PChar(StrPath), 0, SHFileInfo, SizeOf (TSHFileInfo),
    SHGFI_TYPENAME or SHGFI_DISPLAYNAME or SHGFI_SYSICONINDEX or SHGFI_ICON);

  with Info do
  begin
    Icon  := SHFileInfo.hIcon;
    Image := SHFileInfo.iIcon;
    DisplayName := SHFileInfo.szDisplayName;
    TypeName := SHFileInfo.szTypeName;
  end;

  FindFirst(StrPath, 0, SearchRec);
  with Info do
  begin
    try
      DateTime := FileDateToDateTime(SearchRec.Time);
    except
      DateTime := Now();
    end;

    AttrReadOnly := ((SearchRec.Attr and faReadOnly) > 0);
    AttrSystem := ((SearchRec.Attr and faSysFile) > 0);
    AttrHidden := ((SearchRec.Attr and faHidden) > 0);
    AttrArchive := ((SearchRec.Attr and faArchive) > 0);
    AttrVolume := ((SearchRec.Attr and faVolumeID) > 0);
    AttrDirectory := ((SearchRec.Attr and faDirectory) > 0);

    Size := SearchRec.Size;

    SizeDescription := scGetSizeDescription(Size);
  end;
end;

function TVideoForm.ReadMWord(f: TFileStream): Word;
type
  TMotorolaWord = record
    case Byte of
      0: (Value: Word);
      1: (Byte1, Byte2: Byte);
  end;
var
  MW: TMotorolaWord;
begin
  { It would probably be better to just read these two bytes in normally }
  { and then do a small ASM routine to swap them.  But we aren't talking }
  { about reading entire files, so I doubt the performance gain would be }
  { worth the trouble. }
  f.read(MW.Byte2, SizeOf(Byte));
  f.read(MW.Byte1, SizeOf(Byte));
  Result := MW.Value;
end;

procedure TVideoForm.GetJPGSize(const sFile: string; var wWidth, wHeight: Word);
const
  ValidSig: array[0..1] of Byte = ($FF, $D8);
  Parameterless = [$01, $D0, $D1, $D2, $D3, $D4, $D5, $D6, $D7];
var
  Sig: array[0..1] of byte;
  f: TFileStream;
  x: integer;
  Seg: byte;
  Dummy: array[0..15] of byte;
  Len: word;
  ReadLen: LongInt;
begin
  FillChar(Sig, SizeOf(Sig), #0);
  f := TFileStream.Create(sFile, fmOpenRead);
  try
    ReadLen := f.read(Sig[0], SizeOf(Sig));

    for x := Low(Sig) to High(Sig) do
      if Sig[x] <> ValidSig[x] then ReadLen := 0;

    if ReadLen > 0 then
    begin
      ReadLen := f.read(Seg, 1);
      while (Seg = $FF) and (ReadLen > 0) do
      begin
        ReadLen := f.read(Seg, 1);
        if Seg <> $FF then
        begin
          if (Seg = $C0) or (Seg = $C1) then
          begin
            ReadLen := f.read(Dummy[0], 3); { don't need these bytes }
            wHeight := ReadMWord(f);
            wWidth  := ReadMWord(f);
          end 
          else 
          begin
            if not (Seg in Parameterless) then
            begin
              Len := ReadMWord(f);
              f.Seek(Len - 2, 1);
              f.read(Seg, 1);
            end 
            else
              Seg := $FF; { Fake it to keep looping. }
          end;
        end;
      end;
    end;
  finally
    f.Free;
  end;
end;

end.
